/*
 * Print some info about executable files.
 *
 * By JaDa, 7-Oct-1987
 */


#include <stdio.h>
#include <fcntl.h>
#include <a.out.h>

#ifdef TRACEON
#   define TDEV stderr
#   define TP(z) {fprintf/**/z;fflush(TDEV);}
#else
#   define TP(z)
#endif


char *progname;

main(argc, argv)
    int argc;
    char **argv;
{
    FILE *i = stdin, *o = stdout, *e = stderr;
    char *fname;
    int estat = 0;

    progname = *argv++;

    while ((fname = *argv++) != NULL) {
	int fs = open(fname, O_RDONLY, 0);
	if (fs < 0) {
	    fprintf(e, "%s: Can't open file %s : ", progname, fname);
	    perror("");
	    estat++;
	} else {
	    print_info(fname, fs, o);
	    close(fs);
	}
    }

    exit(estat);
}


char *
xcpu(n)
    char n;
{
    static char buff[128];
    char *p = buff;

    *p = '\000';

    switch (n&XC_CPU) {
      case XC_NONE:
	strcpy(p, "no CPU?");
	break;
      case XC_PDP11:
      case XC_23:
      case XC_Z8K:
      case XC_8086:
      case XC_Z80:
      case XC_VAX:
      case XC_32016:
      case XC_286:
	strcpy(p, "by a new CPU!!");
	break;
      case XC_68K:
	strcpy(p, "MC680xx");
	break;
      default:
	strcpy(p, "?");
    }

    p += strlen(p);

    if ((n & XC_BSWAP) == 0) {
	strcpy(p, " BSWAP");
	p += strlen(p);
    }
    if ((n & XC_WSWAP) != 0) {
	strcpy(p, " WSWAP");
	p += strlen(p);
    }

    TP((TDEV, "\r\nxcpu: %s", buff))
    return buff;
}


char *
xtables(rs)
    char rs;
{
    static char buff[256];
    char *p = buff;
    *p = '\000';

    strcpy(p, "Relocation: ");
    p += strlen(p);

    switch (rs & XR_REL) {	/* relocation table format */
      case XR_RXOUT:
	strcpy(p, "linkable");
	p += strlen(p);
	break;
      case XR_RXEXEC:
	strcpy(p, "executable");
	p += strlen(p);
	break;
      case XR_RBOUT:
	strcpy(p, "b.out");
	p += strlen(p);
	break;
      case XR_RAOUT:
	strcpy(p, "a.out");
	p += strlen(p);
	break;
      case XR_R86REL:
	strcpy(p, "8086 relocatable");
	p += strlen(p);
	break;
      case XR_R86ABS:
	strcpy(p, "8086 absolute");
	p += strlen(p);
	break;
      case XR_R286ABS:
	strcpy(p, "80286 absolute");
	p += strlen(p);
	break;
      default:
	break;
    }

    strcpy(p, ", Symbol table: ");
    p += strlen(p);

    switch (rs & XR_SYM) {	/* symbol table format */
      case XR_SXOUT:
	strcpy(p, "trailing string, struct sym");
	p += strlen(p);
	break;
      case XR_SBOUT:
	strcpy(p, "trailing string, struct bsym");
	p += strlen(p);
	break;
      case XR_SAOUT:
	strcpy(p, "struct asym (nlist)");
	p += strlen(p);
	break;
      case XR_S86REL:
	strcpy(p, "8086 relocatable");
	p += strlen(p);
	break;
      case XR_S86ABS:
	strcpy(p, "8086 absolute");
	p += strlen(p);
	break;
      case XR_SUCBVAX:
	strcpy(p, "separate string table");
	p += strlen(p);
	break;
      default:
	strcpy(p, "?");
	p += strlen(p);
    }

    TP((TDEV, "\r\nxtables: %s", buff))
    return buff;
}


char *
xrunenv(xr)
    unsigned short xr;
{
    static char buff[256];
    int flag = 0;
    char *p = buff;
    *p = '\000';

    switch (xr & XE_VERS) {
      case XE_V2:
	strcpy(p, "<= version 2.3");
	p += strlen(p);
	break;
      case XE_V3:
	strcpy(p, "> version 2.3");
	p += strlen(p);
	break;
      default:
	strcpy(p, "?");
	p += strlen(p);
    }

    strcpy(p, "  <");
    p += strlen(p);

    if (xr & XE_RES) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "reserved");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_LTEXT) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "large model text");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_LDATA) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "large model data");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_OVER) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "text overlay");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_FS) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "fixed stack");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_PURE) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "pure text");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_SEP) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "separate I&D");
	p += strlen(p);
	flag++;
    }

    if (xr & XE_EXEC) {
	if (flag) {
	    strcpy(p, ", ");
	    p += strlen(p);
	}
	strcpy(p, "executable");
	p += strlen(p);
	flag++;
    }

    strcpy(p, ">");
    p += strlen(p);

    TP((TDEV, "\r\nxrunenv: %s", buff))
    return buff;
}


int
print_info(fn, fs, o)
    char *fn;
    int fs;
    FILE *o;
{
    /* NOTE: this code asumes that sizeof(x_magic) == sizeof(xa_magic) !! */
    union xu {
	struct xexec x;
	struct aexec a;
	char dummy[1024];
    } u;
    int nb = read(fs, &u, sizeof(u));

    if (nb < sizeof(u.x.x_magic)) return 0;	/* if not executable, ignore */
    switch (u.x.x_magic) {
      case ARCMAGIC:
	fprintf(o, "%s: is an archive\n", fn);
	return 0;
	break;
      case X_MAGIC:
	if (nb < (sizeof(u.x) + u.x.x_ext)) {
	    fprintf(o, "%s: x.out file ?, premature EOF\n", fn);
	    return -1;
	}
	fprintf(o, "%s: x.out header\n", fn);
	fprintf(o, "    Size: text=%d, data=%d, bss=%d, symtab=%d\n",
		u.x.x_text, u.x.x_data, u.x.x_bss, u.x.x_syms);
	fprintf(o, "    Entry: 0x%08lX, cpu = %s\n",
		u.x.x_entry, xcpu(u.x.x_cpu));
	fprintf(o, "    Format: %s\n", xtables(u.x.x_relsym));
	fprintf(o, "    Environ: %s\n", xrunenv(u.x.x_renv));
	break;
      case FMAGIC:
	fprintf(o, "%s: FMAGIC\n", fn);
	break;
      case NMAGIC:
	fprintf(o, "%s: NMAGIC\n", fn);
	break;
      case IMAGIC:
	fprintf(o, "%s: IMAGIC\n", fn);
	break;
      case OMAGIC:
	fprintf(o, "%s: OMAGIC\n", fn);
	break;
      case ZMAGIC:
	fprintf(o, "%s: ZMAGIC\n", fn);
	break;
      default:
	fprintf(o, "%s: not an executable\n", fn);
    }

    return 0;
}

